package apparat.embedding.maven;

import apparat.tools.tdsi.TDSIConfiguration;
import apparat.tools.tdsi.TurboDieselSportInjection;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;

import java.io.File;

/**
 * @author Joa Ebert
 * @phase package
 * @goal tdsi
 * @threadSafe
 */
public final class TurboDieselSportInjectionMojo extends AbstractApparatMojo {
	/**
	 * Whether or not to expand Alchemy instructions.
	 *
	 * @parameter default-value="true" expression="${apparat.tdsi.alchemyExpansion}
	 * @required
	 */
	private boolean alchemyExpansion;

	/**
	 * Whether or not to perform macro expansion.
	 *
	 * @parameter default-value="true" expression="${apparat.tdsi.macroExpansion}
	 * @required
	 */
	private boolean macroExpansion;

	/**
	 * Whether or not to perform inline expansion.
	 *
	 * @parameter default-value="true" expression="${apparat.tdsi.inlineExpansion}
	 * @required
	 */
	private boolean inlineExpansion;

	/**
	 * Whether or not to fix bytecode generated by Alchemy.
	 *
	 * @parameter default-value="false" expression="${apparat.tdsi.fixAlchemy}
	 * @required
	 */
	private boolean fixAlchemy;

	/**
	 * Whether or not to perform ASM expansion.
	 *
	 * @parameter default-value="true" expression="${apparat.tdsi.asmExpansion}
	 * @required
	 */
	private boolean asmExpansion;

	@Override public void execute() throws MojoExecutionException, MojoFailureException {
		if(!alchemyExpansion && !macroExpansion && !inlineExpansion && !fixAlchemy && !asmExpansion) {
			getLog().warn("TurboDieselSportInjection has been disabled since all its " +
					"features are turned off.");
			return;
		}

		super.execute();
	}

	@Override protected void processFile(final File file) {
		if(getLog().isDebugEnabled()) {
			getLog().debug("Running "+file+" through TurboDieselSportInjection ...");
		}

		final TurboDieselSportInjection.TDSITool tool = new TurboDieselSportInjection.TDSITool();
		final TDSIConfiguration config = new TDSIConfiguration() {
			@Override public File input() { return file; }
			@Override public File output() { return file;}
			@Override public boolean alchemyExpansion() { return alchemyExpansion; }
			@Override public boolean macroExpansion() { return macroExpansion; }
			@Override public boolean inlineExpansion() { return inlineExpansion; }
			@Override public boolean fixAlchemy() { return fixAlchemy; }
			@Override public boolean asmExpansion() { return asmExpansion; }
		};

		tool.configure(config);
		tool.run();
	}
}
